@@ -123,7 +123,7 @@ static int build_tone(char *data, int size, struct tone_zone_sound *t, int *coun
 			return -1;
 		}
 #if 0
-		printf("Using %d samples for %d and %d\n", samples, freq1, freq2);
+		printf("Using %d samples for %d and %d\n", time * 8, freq1, freq2);
 #endif
 		if (size < sizeof(struct zt_tone_def)) {
 			fprintf(stderr, "Not enough space for samples\n");
@@ -164,6 +164,9 @@ static int build_tone(char *data, int size, struct tone_zone_sound *t, int *coun
 		/* If we don't end on a solid tone, return */
 		td->next = firstnobang;
 	}
+	if (firstnobang < 0)
+		fprintf(stderr, "tone '%s' does not end with a solid tone or silence (all tone components have an exclamation mark)\n", t->data);
+
 	return used;
 }
 
