@@ -24,7 +24,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
  *
- * $Id: tor2.c,v 1.19.2.3 2005/01/17 01:58:09 russell Exp $
+ * $Id: tor2.c 837 2005-11-29 18:42:08Z kpfleming $
  */
 
 #include <linux/kernel.h>
@@ -56,6 +56,10 @@
 
 /* #define ENABLE_TASKLETS */
 
+/* this stuff needs to work for 64 bit systems, however using the macro causes
+   it to take twice as long */
+/* #define FIXTHISFOR64 */  /* as of now, un-comment for 32 bit only system */
+
 #define SPANS_PER_CARD  4
 #define MAX_SPANS       16
 
@@ -97,7 +101,7 @@ struct tor2 {
 	volatile unsigned short *plx;	/* Virtual representation of local space */
 	unsigned long xilinx32_region;	/* 32 bit Region allocated to Xilinx */
 	unsigned long xilinx32_len;	/* Length of 32 bit Xilinx region */
-	volatile unsigned long *mem32;	/* Virtual representation of 32 bit Xilinx memory area */
+	volatile unsigned int *mem32;	/* Virtual representation of 32 bit Xilinx memory area */
 	unsigned long xilinx8_region;	/* 8 bit Region allocated to Xilinx */
 	unsigned long xilinx8_len;	/* Length of 8 bit Xilinx region */
 	volatile unsigned char *mem8;	/* Virtual representation of 8 bit Xilinx memory area */
@@ -134,6 +138,8 @@ static void tor2_tasklet(unsigned long data);
 #endif
 
 #define	GPIOC (PLX_LOC_GPIOC >> 1) /* word-oriented address for PLX GPIOC reg. (32 bit reg.) */
+#define	LAS2BRD (0x30 >> 1)
+#define	LAS3BRD (0x34 >> 1)
 #define	INTCSR (0x4c >> 1)  /* word-oriented address for PLX INTCSR reg. */
 #define	PLX_INTENA 0x43 /* enable, hi-going, level trigger */
 
@@ -359,8 +365,8 @@ static int __devinit tor2_probe(struct pci_dev *pdev, const struct pci_device_id
 	int res,x,f;
 	struct tor2 *tor;
 	unsigned long endjif;
-	volatile unsigned long *gpdata_io;
-	unsigned long gpdata;
+	volatile unsigned long *gpdata_io,*lasdata_io;
+	unsigned long gpdata,lasdata;
 
 	res = pci_enable_device(pdev);
 	if (res)
@@ -511,6 +517,19 @@ static int __devinit tor2_probe(struct pci_dev *pdev, const struct pci_device_id
 	tor->mem8[CTLREG1] = 0;
 	tor->mem8[LEDREG] = 0;
 
+	/* set the LA2BRD register so that we enable block transfer, read
+	   pre-fetch, and set to maximum read pre-fetch size */
+	lasdata_io = (unsigned long *)&tor->plx[LAS2BRD];
+	lasdata = *lasdata_io;
+	lasdata |= 0x39;
+	*lasdata_io = lasdata;
+
+	/* set the LA3BRD register so that we enable block transfer */
+	lasdata_io = (unsigned long *)&tor->plx[LAS3BRD];
+	lasdata = *lasdata_io;
+	lasdata |= 1;
+	*lasdata_io = lasdata;
+
 	/* check part revision data */
 	x = t1in(tor,1,0xf) & 15;
 #ifdef	NONREVA
@@ -628,7 +647,7 @@ static struct pci_driver tor2_driver = {
 
 static int __init tor2_init(void) {
 	int res;
-	res = pci_module_init(&tor2_driver);
+	res = zap_pci_module(&tor2_driver);
 	printk("Registered Tormenta2 PCI\n");
 	return res;
 }
@@ -1087,7 +1106,11 @@ static void tor2_tasklet(unsigned long data)
 static int syncsrc = 0;
 static int syncnum = 0 /* -1 */;
 static int syncspan = 0;
+#ifdef DEFINE_SPINLOCK
+static DEFINE_SPINLOCK(synclock);
+#else
 static spinlock_t synclock = SPIN_LOCK_UNLOCKED;
+#endif
 
 static int tor2_findsync(struct tor2 *tor)
 {
@@ -1172,7 +1195,7 @@ static void tor2_intr(int irq, void *dev_id, struct pt_regs *regs)
 #endif
 {
 	int n, i, j, k, syncsrc;
-	unsigned long rxword,txword;
+	unsigned int rxword,txword;
 
 	unsigned char c, rxc;
 	unsigned char abits, bbits;
@@ -1212,7 +1235,11 @@ static void tor2_intr(int irq, void *dev_id, struct pt_regs *regs)
 			/* span 4 */
 			txword |= tor->spans[3].chans[n].writechunk[i];
 			/* write to part */
+#ifdef FIXTHISFOR64
+			tor->mem32[tor->datxlt[n] + (32 * i)] = txword;
+#else
 			tor->mem32[tor->datxlt[n] + (32 * i)] = cpu_to_le32(txword);
+#endif
 		}
 	}
 
@@ -1220,7 +1247,11 @@ static void tor2_intr(int irq, void *dev_id, struct pt_regs *regs)
 	for (n = 0; n < tor->spans[0].channels; n++) {
 		for (i = 0; i < ZT_CHUNKSIZE; i++) {
 			/* read from */
+#ifdef FIXTHISFOR64
+		        rxword = tor->mem32[tor->datxlt[n] + (32 * i)];
+#else
 			rxword = le32_to_cpu(tor->mem32[tor->datxlt[n] + (32 * i)]);
+#endif
 			/* span 1 */
 			tor->spans[0].chans[n].readchunk[i] = rxword >> 24;
 			/* span 2 */
