@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
  *
- * $Id: wct1xxp.c,v 1.19.2.3 2005/01/17 01:58:09 russell Exp $
+ * $Id: wct1xxp.c 837 2005-11-29 18:42:08Z kpfleming $
  */
 
 #include <linux/kernel.h>
@@ -137,7 +137,7 @@ struct t1xxp {
 	/* Our offset for finding channel 1 */
 	int offset;
 	char *variety;
-	int intcount;
+	unsigned int intcount;
 	int usecount;
 	int clocktimeout;
 	int sync;
@@ -791,7 +791,7 @@ static int t1xxp_software_init(struct t1xxp *wc)
 		sprintf(wc->chans[x].name, "WCT1/%d/%d", wc->num, x + 1);
 		wc->chans[x].sigcap = ZT_SIG_EM | ZT_SIG_CLEAR | ZT_SIG_EM_E1 | 
 				      ZT_SIG_FXSLS | ZT_SIG_FXSGS | 
-				      ZT_SIG_FXSKS | ZT_SIG_FXOLS | 
+				      ZT_SIG_FXSKS | ZT_SIG_FXOLS | ZT_SIG_DACS_RBS |
 				      ZT_SIG_FXOGS | ZT_SIG_FXOKS | ZT_SIG_CAS | ZT_SIG_SF;
 		wc->chans[x].pvt = wc;
 		wc->chans[x].chanpos = x + 1;
@@ -1390,7 +1390,7 @@ static struct pci_driver t1xxp_driver = {
 static int __init t1xxp_init(void)
 {
 	int res;
-	res = pci_module_init(&t1xxp_driver);
+	res = zap_pci_module(&t1xxp_driver);
 	if (res)
 		return -ENODEV;
 	return 0;
