@@ -1184,10 +1184,10 @@ static int wctdm_proslic_insane(struct wctdm *wc, int card)
 		printk("ProSLIC 3210 version %d is too old\n", blah & 0xf);
 		return -1;
 	}
-	if ((blah & 0xf) == 2) {
+	if (wctdm_getreg(wc, card, 1) & 0x80)
 		/* ProSLIC 3215, not a 3210 */
 		wc->flags[card] |= FLAG_3215;
-	}
+
 	blah = wctdm_getreg(wc, card, 8);
 	if (blah != 0x2) {
 		printk("ProSLIC on module %d insane (1) %d should be 2\n", card, blah);
