@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
  *
- * $Id: wcte11xp.c 837 2005-11-29 18:42:08Z kpfleming $
+ * $Id: wcte11xp.c 1573 2006-11-08 23:04:33Z kpfleming $
  */
 
 #include <linux/kernel.h>
@@ -43,7 +43,8 @@
 #include <linux/moduleparam.h>
 #endif
 
-#include "wct4xxp.h"	/* For certain definitions */
+/* XXX: fix this */
+#include "wct4xxp/wct4xxp.h"	/* For certain definitions */
 
 #define WC_MAX_CARDS	32
 
@@ -615,7 +616,7 @@ static void t4_serial_setup(struct t1 *wc)
 {
 	printk("TE110P: Setting up global serial parameters for %s %s\n", 
 	       wc->spantype == TYPE_E1 ? "E1" : "T1", 
-		   wc->spanflags & FLAG_FALC12 ? "FALC V1.2" : "FALC V2.1");
+		   wc->spanflags & FLAG_FALC12 ? "FALC V1.2" : "FALC V2.2");
 	t1_framer_out(wc, 0x85, 0xe0);	/* GPC1: Multiplex mode enabled, FSC is output, active low, RCLK from channel 0 */
 	t1_framer_out(wc, 0x08, 0x05);	/* IPC: Interrupt push/pull active low */
 	if (wc->spanflags & FLAG_FALC12) {
@@ -626,38 +627,15 @@ static void t4_serial_setup(struct t1 *wc)
 		t1_framer_out(wc, 0x96, 0x03);
 		t1_framer_out(wc, 0x97, 0x10);
 	} else {
-#ifdef TRUST_INFINEON_ERRATA
-		if (wc->spantype == TYPE_E1) {
-			/* Global clocks (8.192 Mhz CLK) */
-			t1_framer_out(wc, 0x92, 0x00);	
-			t1_framer_out(wc, 0x93, 0x00);
-			t1_framer_out(wc, 0x94, 0x00);
-			t1_framer_out(wc, 0x95, 0x00);
-			t1_framer_out(wc, 0x96, 0x00);
-			t1_framer_out(wc, 0x97, 0x0F);
-			t1_framer_out(wc, 0x98, 0x80);
-			t1_framer_out(wc, 0x99, 0x00);
-		} else {	
-			/* Global clocks (8.192 Mhz CLK) */
-			t1_framer_out(wc, 0x92, 0x00);	
-			t1_framer_out(wc, 0x93, 0x10);
-			t1_framer_out(wc, 0x94, 0xfb);
-			t1_framer_out(wc, 0x95, 0x03);
-			t1_framer_out(wc, 0x96, 0x00);
-			t1_framer_out(wc, 0x97, 0x0b);
-			t1_framer_out(wc, 0x98, 0x8c);
-			t1_framer_out(wc, 0x99, 0x80);
-		}
-#else
-		t1_framer_out(wc, 0x92, 0x66);	
-		t1_framer_out(wc, 0x93, 0x0e);
-		t1_framer_out(wc, 0x94, 0x3f);
-		t1_framer_out(wc, 0x95, 0x0f);
-		t1_framer_out(wc, 0x96, 0x04);
-		t1_framer_out(wc, 0x97, 0x3c);
-		t1_framer_out(wc, 0x98, 0x9c);
-		t1_framer_out(wc, 0x99, 0x90);		
-#endif
+		/* Global clocks (8.192 Mhz CLK) */
+		t1_framer_out(wc, 0x92, 0x00);	
+		t1_framer_out(wc, 0x93, 0x18);
+		t1_framer_out(wc, 0x94, 0xfb);
+		t1_framer_out(wc, 0x95, 0x0b);
+		t1_framer_out(wc, 0x96, 0x00);
+		t1_framer_out(wc, 0x97, 0x0b);
+		t1_framer_out(wc, 0x98, 0xdb);
+		t1_framer_out(wc, 0x99, 0xdf);
 	}
 	/* Configure interrupts */	
 	t1_framer_out(wc, 0x46, 0x40);	/* GCR: Interrupt on Activation/Deactivation of AIX, LOS */
@@ -674,7 +652,7 @@ static void t4_serial_setup(struct t1 *wc)
 	if (wc->spanflags & FLAG_FALC12)
 		t1_framer_out(wc, 0x25, 0x04);	/* RC1: The rest of RC0 */
 	else
-		t1_framer_out(wc, 0x25, 0x06);	/* RC1: The rest of RC0 */
+		t1_framer_out(wc, 0x25, 0x05);	/* RC1: The rest of RC0 */
 	
 	/* Configure ports */
 	t1_framer_out(wc, 0x80, 0x00);	/* PC1: SPYR/SPYX input on RPA/XPA */
