@@ -31,7 +31,7 @@
 #include "card_fxo.h"
 #include "zap_debug.h"
 
-static const char rcsid[] = "$Id: card_fxo.c 1230 2006-07-14 20:19:48Z tzafrir $";
+static const char rcsid[] = "$Id: card_fxo.c 1457 2006-09-09 15:24:12Z tzafrir $";
 
 DEF_PARM(int, print_dbg, 0, "Print DBG statements");	/* must be before zap_debug.h */
 DEF_PARM(uint, poll_battery_interval, 100, "Poll battery interval in milliseconds");
@@ -51,12 +51,14 @@ enum fxo_leds {
 #define	NUM_LEDS		1
 #define	DELAY_UNTIL_DIALTONE	3000
 
+#define	BAT_THRESHOLD		3
+#define	BAT_DEBOUNCE		3	/* compensate for battery voltage fluctuation (in poll_battery_interval's) */
+
 /*---------------- FXO Protocol Commands ----------------------------------*/
 
 static /* 0x0F */ DECLARE_CMD(FXO, CHAN_ENABLE, xpp_line_t lines, bool on);
 static /* 0x0F */ DECLARE_CMD(FXO, CHAN_CID, int pos);
 static /* 0x0F */ DECLARE_CMD(FXO, RING, int pos, bool on);
-static /* 0x0F */ DECLARE_CMD(FXO, SETHOOK, int pos, bool offhook);
 static /* 0x0F */ DECLARE_CMD(FXO, RELAY_OUT, byte which, bool on);
 static /* 0x0F */ DECLARE_CMD(FXO, DAA_QUERY, int pos, byte reg_num);
 
@@ -70,20 +72,49 @@ static int process_slic_cmdline(xpd_t *xpd, char *cmdline);
 #define	PROC_DAA_FNAME		"slics"
 #define	PROC_FXO_INFO_FNAME	"fxo_info"
 
+#ifdef SOFT_RING
+#define	POLL_RING_INTERVAL	2
+#define	RING_THRESHOLD		3
+#define	NORING_THRESHOLD	10
+#endif
+#define	DAA_RING_REGISTER	0x05
+
 struct FXO_priv_data {
 	struct proc_dir_entry		*xpd_slic;
 	struct proc_dir_entry		*fxo_info;
+	uint				poll_counter;
 	slic_reply_t			requested_reply;
 	slic_reply_t			last_reply;
 	xpp_line_t			battery;
+	ushort				battery_debounce[CHANNELS_PERXPD];
 	xpp_line_t			ledstate[NUM_LEDS];	/* 0 - OFF, 1 - ON */
+	xpp_line_t			ledcontrol[NUM_LEDS];	/* 0 - OFF, 1 - ON */
 	int				blinking[NUM_LEDS][CHANNELS_PERXPD];
+#ifdef SOFT_RING
+	ushort				ring_thresh[CHANNELS_PERXPD];
+	ushort				noring_thresh[CHANNELS_PERXPD];
+	/* ring_sig is set when Reg5, bit 2 (Ring Detect) is set. 
+	 * While ring_sig=1 we check R5 bit 20H and 40H for ringing. 
+	 * When it drops to 0 that's the end of the ring sequence and 
+	 * we clear the ring detection variables */
+	ushort				ring_sig[CHANNELS_PERXPD];
+#endif
 };
 
 /*---------------- FXO: Static functions ----------------------------------*/
 
 #define	IS_BLINKING(priv,pos,color)	((priv)->blinking[color][pos] != 0)
-#define	DO_BLINK(priv,pos,color,val)	((priv)->blinking[color][pos] = (val))
+#define	MARK_BLINK(priv,pos,color,val)	((priv)->blinking[color][pos] = (val))
+
+void MARK_LED(xpd_t *xpd, lineno_t pos, byte color, bool on)
+{
+	struct FXO_priv_data *priv = xpd->priv;
+
+	if(on)
+		BIT_SET(priv->ledcontrol[color], pos);
+	else
+		BIT_CLR(priv->ledcontrol[color], pos);
+}
 
 /*
  * LED control is done via DAA register 0x20
@@ -130,10 +161,10 @@ out:
 
 static void handle_fxo_leds(xpd_t *xpd)
 {
-	int		i;
-	unsigned long	flags;
+	int			i;
+	unsigned long		flags;
 	const enum fxo_leds	color = LED_GREEN;
-	unsigned int	timer_count;
+	unsigned int		timer_count;
 	struct FXO_priv_data	*priv;
 
 	BUG_ON(!xpd);
@@ -146,7 +177,7 @@ static void handle_fxo_leds(xpd_t *xpd)
 		if(IS_BLINKING(priv,i,color)) {
 			// led state is toggled
 			if((timer_count % LED_BLINK_PERIOD) == 0) {
-				DBG("%s/%s/%d: led_state=%s\n", xpd->xbus->busname, xpd->xpdname, i,
+				DBG("%s/%s/%d: ledstate=%s\n", xpd->xbus->busname, xpd->xpdname, i,
 						(IS_SET(priv->ledstate[color], i))?"ON":"OFF");
 				if(!IS_SET(priv->ledstate[color], i)) {
 					do_led(xpd, i, color, 1);
@@ -154,35 +185,78 @@ static void handle_fxo_leds(xpd_t *xpd)
 					do_led(xpd, i, color, 0);
 				}
 			}
+		} else if(IS_SET(priv->ledcontrol[color], i) && !IS_SET(priv->ledstate[color], i)) {
+			do_led(xpd, i, color, 1);
+		} else if(!IS_SET(priv->ledcontrol[color], i) && IS_SET(priv->ledstate[color], i)) {
+			do_led(xpd, i, color, 0);
 		}
 	}
 	spin_unlock_irqrestore(&xpd->lock, flags);
 }
 
-static void do_sethook(xpd_t *xpd, int pos, bool offhook)
+static void mark_ring(xpd_t *xpd, lineno_t pos, bool on)
+{
+	struct FXO_priv_data	*priv;
+
+	priv = xpd->priv;
+	BUG_ON(!priv);
+	if(on && !xpd->ringing[pos]) {
+		DBG("%s/%s/%d: START\n", xpd->xbus->busname, xpd->xpdname, pos);
+		xpd->ringing[pos] = 1;
+		MARK_BLINK(priv, pos, LED_GREEN, LED_BLINK);
+		update_zap_ring(xpd, pos, 1);
+	} else if(!on && xpd->ringing[pos]) {
+		DBG("%s/%s/%d: STOP\n", xpd->xbus->busname, xpd->xpdname, pos);
+		xpd->ringing[pos] = 0;
+		if(IS_BLINKING(priv, pos, LED_GREEN))
+			MARK_BLINK(priv, pos, LED_GREEN, 0);
+		update_zap_ring(xpd, pos, 0);
+	}
+}
+
+static int do_sethook(xpd_t *xpd, int pos, bool to_offhook)
 {
 	unsigned long		flags;
+	xbus_t			*xbus;
 	struct FXO_priv_data	*priv;
+	int			ret = 0;
+	bool			value;
+	xpacket_t		*pack;
+	slic_cmd_t		*sc;
+	int			len;
 
 	BUG_ON(!xpd);
 	BUG_ON(xpd->direction == TO_PHONE);		// We can SETHOOK state only on PSTN
+	xbus = xpd->xbus;
 	priv = xpd->priv;
 	BUG_ON(!priv);
 	if(!IS_SET(priv->battery, pos)) {
-		DBG("%s/%s/%d: WARNING: called while battery is off\n", xpd->xbus->busname, xpd->xpdname, pos);
+		NOTICE("%s/%s/%d: WARNING: called while battery is off\n", xbus->busname, xpd->xpdname, pos);
 	}
 	spin_lock_irqsave(&xpd->lock, flags);
-	xpd->ringing[pos] = 0;				// No more rings
-	CALL_XMETHOD(SETHOOK, xpd->xbus, xpd, pos, offhook);
-	if(offhook) {
-		BIT_SET(xpd->hookstate, pos);
+	mark_ring(xpd, pos, 0);				// No more rings
+	value = (to_offhook) ? 0x09 : 0x08;	/* Bit 3 is for CID */
+	DBG("%s/%s/%d: SETHOOK: value=0x%02X %s\n", xbus->busname, xpd->xpdname, pos, value, (to_offhook)?"OFFHOOK":"ONHOOK");
+	MARK_LED(xpd, pos, LED_GREEN, (to_offhook)?LED_ON:LED_OFF);
+	XPACKET_NEW(pack, xbus, FXO, DAA_WRITE, xpd->id);
+	sc = &RPACKET_FIELD(pack, FXO, DAA_WRITE, slic_cmd);
+	len = slic_cmd_direct_write(sc, BIT(pos), DAA_RING_REGISTER, value);
+	pack->datalen = len;
+	packet_send(xbus, pack);
+#ifdef	SOFT_RING
+	priv->ring_sig[pos] = 0;
+#endif
+	if(to_offhook) {
+		BIT_SET(xpd->offhook, pos);
 	} else {
-		BIT_CLR(xpd->hookstate, pos);
+		BIT_CLR(xpd->offhook, pos);
+		BIT_CLR(xpd->cid_on, pos);
 		xpd->delay_until_dialtone[pos] = 0;
 	}
 	spin_unlock_irqrestore(&xpd->lock, flags);
-	if(offhook)
+	if(to_offhook)
 		wake_up_interruptible(&xpd->txstateq[pos]);
+	return ret;
 }
 
 /*---------------- FXO: Methods -------------------------------------------*/
@@ -282,7 +356,6 @@ static int FXO_card_zaptel_preregistration(xpd_t *xpd, bool on)
 	xbus_t			*xbus;
 	struct FXO_priv_data	*priv;
 	int			i;
-	unsigned long		flags;
 
 	BUG_ON(!xpd);
 	xbus = xpd->xbus;
@@ -300,11 +373,9 @@ static int FXO_card_zaptel_preregistration(xpd_t *xpd, bool on)
 		cur_chan->pvt = xpd;
 		cur_chan->sigcap = FXO_DEFAULT_SIGCAP;
 	}
-	spin_lock_irqsave(&xpd->lock, flags);
-	do_led(xpd, ALL_LINES, LED_GREEN, LED_OFF);
-	spin_unlock_irqrestore(&xpd->lock, flags);
+	MARK_LED(xpd, ALL_LINES, LED_GREEN, LED_OFF);
 	for_each_line(xpd, i) {
-		do_led(xpd, i, LED_GREEN, LED_ON);
+		MARK_LED(xpd, i, LED_GREEN, LED_ON);
 		mdelay(50);
 	}
 	return 0;
@@ -323,13 +394,12 @@ static int FXO_card_zaptel_postregistration(xpd_t *xpd, bool on)
 	BUG_ON(!priv);
 	DBG("%s/%s (%d)\n", xbus->busname, xpd->xpdname, on);
 	for_each_line(xpd, i) {
-		do_led(xpd, i, LED_GREEN, LED_OFF);
+		MARK_LED(xpd, i, LED_GREEN, LED_OFF);
 		mdelay(50);
 	}
 	return 0;
 }
 
-#ifdef WITH_RBS
 int FXO_card_hooksig(xbus_t *xbus, xpd_t *xpd, int pos, zt_txsig_t txsig)
 {
 	struct FXO_priv_data	*priv;
@@ -355,43 +425,6 @@ int FXO_card_hooksig(xbus_t *xbus, xpd_t *xpd, int pos, zt_txsig_t txsig)
 	return 0;
 }
 
-#else
-int FXO_card_sethook(xbus_t *xbus, xpd_t *xpd, int pos, int hookstate)
-{
-	int			ret = 0;
-	struct FXO_priv_data	*priv;
-
-	DBG("%s/%s/%d: %s\n", xbus->busname, xpd->xpdname, pos, hookstate2str(hookstate));
-	switch(hookstate) {
-		/* On-hook, off-hook: The PBX is playing a phone on an FXO line.  */
-		case ZT_ONHOOK:
-			do_sethook(xpd, pos, 0);
-			break;
-		case ZT_START:
-			DBG("%s/%s/%d: fall through ZT_OFFHOOK\n", xbus->busname, xpd->xpdname, pos);
-			xpd->delay_until_dialtone[pos] = DELAY_UNTIL_DIALTONE;
-			// Fall through
-		case ZT_OFFHOOK:
-			do_sethook(xpd, pos, 1);
-			wait_event_interruptible(xpd->txstateq[pos], xpd->delay_until_dialtone[pos] <= 0);
-			break;
-		case ZT_WINK:
-			WARN("No code yet\n");
-			break;
-		case ZT_FLASH:
-			WARN("No code yet\n");
-			break;
-		case ZT_RING:
-			DBG("%s/%s/%d: ZT_RING: %d\n", xbus->busname, xpd->xpdname, pos, xpd->ringing[pos]);
-			break;
-		case ZT_RINGOFF:
-			WARN("No code yet\n");
-			break;
-	}
-	return ret;
-}
-#endif
-
 static void poll_battery(xbus_t *xbus, xpd_t *xpd)
 {
 	int	i;
@@ -401,20 +434,37 @@ static void poll_battery(xbus_t *xbus, xpd_t *xpd)
 	}
 }
 
+#ifdef	SOFT_RING
+static void poll_ring(xbus_t *xbus, xpd_t *xpd)
+{
+	int			i;
+	struct FXO_priv_data	*priv;
+
+	priv = xpd->priv;
+	BUG_ON(!priv);
+	for_each_line(xpd, i) {
+		if(priv->ring_sig[i])
+			CALL_PROTO(FXO, DAA_QUERY, xbus, xpd, i, DAA_RING_REGISTER);
+	}
+}
+#endif
 
 static int FXO_card_tick(xbus_t *xbus, xpd_t *xpd)
 {
-	static unsigned		rate_limit = 0;
 	struct FXO_priv_data	*priv;
 
 	BUG_ON(!xpd);
 	priv = xpd->priv;
 	BUG_ON(!priv);
-	rate_limit++;
-	if(poll_battery_interval != 0 && (rate_limit % poll_battery_interval) == 0) {
+	if(poll_battery_interval != 0 && (priv->poll_counter % poll_battery_interval) == 0) {
 		poll_battery(xbus, xpd);
 	}
+#ifdef	SOFT_RING
+	if((priv->poll_counter % POLL_RING_INTERVAL) == 0)
+		poll_ring(xbus, xpd);
+#endif
 	handle_fxo_leds(xpd);
+	priv->poll_counter++;
 	return 0;
 }
 
@@ -433,13 +483,13 @@ static int FXO_card_ioctl(xpd_t *xpd, int pos, unsigned int cmd, unsigned long a
 	int 		i,ret;
 
 	BUG_ON(!xpd);
-	DBG("cmd: 0x%x, expecting: 0x%x, pos=%d.\n", cmd, WCTDM_SET_ECHOTUNE, pos);
+	DBG("cmd: 0x%X, expecting: 0x%X, pos=%d.\n", cmd, WCTDM_SET_ECHOTUNE, pos);
 	switch (cmd) {
 		case WCTDM_SET_ECHOTUNE:
 			DBG("-- Setting echo registers: \n");
 			/* first off: check if this span is fxs. If not: -EINVALID */
 			if (copy_from_user(&echoregs.wctdm_struct, 
-				(struct wctdm_echo_coefs*)arg, sizeof(echoregs.wctdm_struct)))
+				(struct wctdm_echo_coefs __user *)arg, sizeof(echoregs.wctdm_struct)))
 				return -EFAULT;
 
 			/* Set the ACIM register */
@@ -474,7 +524,6 @@ static int FXO_card_ioctl(xpd_t *xpd, int pos, unsigned int cmd, unsigned long a
 
 static /* 0x0F */ HOSTCMD(FXO, CHAN_ENABLE, xpp_line_t lines, bool on)
 {
-	unsigned long		flags;
 	int	ret = 0;
 	int	i;
 
@@ -486,15 +535,11 @@ static /* 0x0F */ HOSTCMD(FXO, CHAN_ENABLE, xpp_line_t lines, bool on)
 	DBG("Channel Activation: 0x%4X %s\n", lines, (on) ? "on" : "off");
 	if(on) {
 		for_each_line(xpd, i) {
-			spin_lock_irqsave(&xpd->lock, flags);
-			do_led(xpd, i, LED_GREEN, LED_ON);
-			spin_unlock_irqrestore(&xpd->lock, flags);
+			MARK_LED(xpd, i, LED_GREEN, LED_ON);
 			mdelay(20);
 		}
 		for_each_line(xpd, i) {
-			spin_lock_irqsave(&xpd->lock, flags);
-			do_led(xpd, i, LED_GREEN, LED_OFF);
-			spin_unlock_irqrestore(&xpd->lock, flags);
+			MARK_LED(xpd, i, LED_GREEN, LED_OFF);
 			mdelay(20);
 		}
 	}
@@ -539,31 +584,6 @@ static /* 0x0F */ HOSTCMD(FXO, RING, int pos, bool on)
 	return ret;
 }
 
-static /* 0x0F */ HOSTCMD(FXO, SETHOOK, int pos, bool offhook)
-{
-	int		ret = 0;
-	xpacket_t	*pack;
-	slic_cmd_t	*sc;
-	int		len;
-	unsigned long	flags;
-	bool		value;
-
-	BUG_ON(!xbus);
-	BUG_ON(!xpd);
-	value = (offhook) ? 0x09 : 0x08;
-	// value |= BIT(3);	/* Bit 3 is for CID */
-	DBG("%s/%s/%d: SETHOOK: value=0x%02X %s\n", xbus->busname, xpd->xpdname, pos, value, (offhook)?"OFFHOOK":"ONHOOK");
-	spin_lock_irqsave(&xpd->lock, flags);
-	XPACKET_NEW(pack, xbus, FXO, DAA_WRITE, xpd->id);
-	sc = &RPACKET_FIELD(pack, FXO, DAA_WRITE, slic_cmd);
-	len = slic_cmd_direct_write(sc, BIT(pos), 0x05, value);
-	pack->datalen = len;
-	packet_send(xbus, pack);
-	do_led(xpd, pos, LED_GREEN, (offhook)?LED_ON:LED_OFF);
-	spin_unlock_irqrestore(&xpd->lock, flags);
-	return ret;
-}
-
 static /* 0x0F */ HOSTCMD(FXO, RELAY_OUT, byte which, bool on)
 {
 	return -ENOSYS;
@@ -597,19 +617,39 @@ HANDLER_DEF(FXO, SIG_CHANGED)
 	xpp_line_t	sig_toggles = RPACKET_FIELD(pack, FXO, SIG_CHANGED, sig_toggles);
 	unsigned long	flags;
 	int		i;
+	struct FXO_priv_data	*priv;
 
 	if(!xpd) {
 		NOTICE("%s: received %s for non-existing xpd: %d\n",
 				__FUNCTION__, cmd->name, XPD_NUM(pack->content.addr));
 		return -EPROTO;
 	}
+	priv = xpd->priv;
+	BUG_ON(!priv);
 	DBG("%s/%s: (PSTN) sig_toggles=0x%04X sig_status=0x%04X\n", xpd->xbus->busname, xpd->xpdname, sig_toggles, sig_status);
 	spin_lock_irqsave(&xpd->lock, flags);
 	for_each_line(xpd, i) {
-		if(IS_SET(sig_status, i)) {
-			xpd->ringing[i] = 1;
-		} else {
-			xpd->ringing[i] = 0;
+		if(IS_SET(sig_toggles, i)) {
+			if(!IS_SET(priv->battery, i)) {
+				DBG("%s/%s/%d: battery is off. ignore false alarm.\n",
+					xbus->busname, xpd->xpdname, i);
+				continue;
+			}
+			if(IS_SET(sig_status, i)) {
+#ifdef	SOFT_RING
+				priv->ring_sig[i]=1;		/* trigger register polling */
+				/* reset ring check counters */
+				priv->ring_thresh[i] = 0;
+				priv->noring_thresh[i] = 0;
+#else
+				mark_ring(xpd, i, 1);
+#endif
+			} else {
+#ifdef	SOFT_RING
+				priv->ring_sig[i] = 0;
+#endif
+				mark_ring(xpd, i, 0);
+			}
 		}
 	}
 	spin_unlock_irqrestore(&xpd->lock, flags);
@@ -632,25 +672,58 @@ HANDLER_DEF(FXO, DAA_REPLY)
 	priv = xpd->priv;
 	BUG_ON(!priv);
 	if(!info->indirect && info->reg_num == DAA_VBAT_REGISTER) {
-		xpp_line_t	last_batt_on = priv->battery;
-		xpp_line_t	changed_lines;
+		byte		bat = abs((signed char)info->data_low);
 		int		i;
 
-		if(abs(info->data_low) < BAT_THRESHOLD) {
-			priv->battery &= ~lines;
-			// DBG("%s/%s: BATTERY OFF (%04X) = %d\n", xpd->xbus->busname, xpd->xpdname, lines, info->data_low);
-		} else {
-			priv->battery |= lines;
-			// DBG("%s/%s: BATTERY ON (%04X) = %d\n", xpd->xbus->busname, xpd->xpdname, lines, info->data_low);
-		}
-		changed_lines = last_batt_on ^ priv->battery;
 		for_each_line(xpd, i) {
-			if(IS_SET(changed_lines, i)) {
-				update_line_status(xpd, i, IS_SET(priv->battery, i));
+			if(!IS_SET(lines, i))
+				continue;
+			if(bat < BAT_THRESHOLD) {
+				/*
+				 * Check for battery voltage fluctuations
+				 */
+				if(IS_SET(priv->battery, i) && priv->battery_debounce[i]++ > BAT_DEBOUNCE) {
+					DBG("%s/%s: BATTERY OFF (%04X) voltage=%d\n", xpd->xbus->busname, xpd->xpdname, lines, bat);
+					BIT_CLR(priv->battery, i);
+					update_line_status(xpd, i, 0);
+				}
+			} else {
+				priv->battery_debounce[i] = 0;
+				if(!IS_SET(priv->battery, i)) {
+					DBG("%s/%s: BATTERY ON (%04X) voltage=%d\n", xpd->xbus->busname, xpd->xpdname, lines, bat);
+					BIT_SET(priv->battery, i);
+				}
 			}
 		}
 	}
+#ifdef	SOFT_RING
+	if(!info->indirect && info->reg_num == DAA_RING_REGISTER) {
+		bool		ringit = (info->data_low & (0x20 | 0x40)) ? 1 : 0;	/* Ring positive | Ring negative */
+		int		i;
+
+		for_each_line(xpd, i) {
+			if(!IS_SET(lines, i))
+				continue;
+			if (!priv->ring_sig[i])
+				continue;
+			if(ringit) {
+				if(priv->ring_thresh[i] > RING_THRESHOLD) {
+					mark_ring(xpd, i, 1);
+					priv->noring_thresh[i] = 0;
+				} else
+					priv->ring_thresh[i]++;
+			} else {
+				if(priv->noring_thresh[i] > NORING_THRESHOLD) {
+					mark_ring(xpd, i, 0);
+					priv->ring_thresh[i] = 0;
+				} else
+					priv->noring_thresh[i]++;
+			}
+		}
+	}
+#endif
 #if 0
+	if (info->reg_num != 29) 
 	DBG("DAA_REPLY: xpd #%d %s reg_num=0x%X, dataL=0x%X dataH=0x%X\n",
 			xpd->id, (info->indirect)?"I":"D",
 			info->reg_num, info->data_low, info->data_high);
@@ -681,16 +754,11 @@ xproto_table_t PROTO_TABLE(FXO) = {
 		.card_remove	= FXO_card_remove,
 		.card_zaptel_preregistration	= FXO_card_zaptel_preregistration,
 		.card_zaptel_postregistration	= FXO_card_zaptel_postregistration,
-#ifdef WITH_RBS
 		.card_hooksig	= FXO_card_hooksig,
-#else
-		.card_sethook	= FXO_card_sethook,
-#endif
 		.card_tick	= FXO_card_tick,
 		.card_ioctl	= FXO_card_ioctl,
 
 		.RING		= XPROTO_CALLER(FXO, RING),
-		.SETHOOK	= XPROTO_CALLER(FXO, SETHOOK),
 		.RELAY_OUT	= XPROTO_CALLER(FXO, RELAY_OUT),
 		.CHAN_ENABLE	= XPROTO_CALLER(FXO, CHAN_ENABLE),
 		.CHAN_CID	= XPROTO_CALLER(FXO, CHAN_CID),
@@ -734,21 +802,47 @@ static int proc_fxo_info_read(char *page, char **start, off_t off, int count, in
 	len += sprintf(page + len, "\t%-17s: ", "Channel");
 	for_each_line(xpd, i) {
 		if(!IS_SET(xpd->digital_outputs, i) && !IS_SET(xpd->digital_inputs, i))
-			len += sprintf(page + len, "%d ", i % 10);
+			len += sprintf(page + len, "%2d ", i % 10);
 	}
 	len += sprintf(page + len, "\n\t%-17s: ", "ledstate");
 	for_each_line(xpd, i) {
 		if(!IS_SET(xpd->digital_outputs, i) && !IS_SET(xpd->digital_inputs, i))
-			len += sprintf(page + len, "%d ", IS_SET(priv->ledstate[LED_GREEN], i));
+			len += sprintf(page + len, "%2d ", IS_SET(priv->ledstate[LED_GREEN], i));
+	}
+	len += sprintf(page + len, "\n\t%-17s: ", "ledcontrol");
+	for_each_line(xpd, i) {
+		if(!IS_SET(xpd->digital_outputs, i) && !IS_SET(xpd->digital_inputs, i))
+			len += sprintf(page + len, "%2d ", IS_SET(priv->ledcontrol[LED_GREEN], i));
 	}
 	len += sprintf(page + len, "\n\t%-17s: ", "blinking");
 	for_each_line(xpd, i) {
 		if(!IS_SET(xpd->digital_outputs, i) && !IS_SET(xpd->digital_inputs, i))
-			len += sprintf(page + len, "%d ", IS_BLINKING(priv,i,LED_GREEN));
+			len += sprintf(page + len, "%2d ", IS_BLINKING(priv,i,LED_GREEN));
 	}
+#ifdef	SOFT_RING
+	len += sprintf(page + len, "\n\t%-17s: ", "ring_thresh");
+	for_each_line(xpd, i) {
+		if(!IS_SET(xpd->digital_outputs, i) && !IS_SET(xpd->digital_inputs, i))
+			len += sprintf(page + len, "%2d ", priv->ring_thresh[i]);
+	}
+	len += sprintf(page + len, "\n\t%-17s: ", "noring_thresh");
+	for_each_line(xpd, i) {
+		if(!IS_SET(xpd->digital_outputs, i) && !IS_SET(xpd->digital_inputs, i))
+			len += sprintf(page + len, "%2d ", priv->noring_thresh[i]);
+	}
+	len += sprintf(page + len, "\n\t%-17s: ", "ring_sig");
+	for_each_line(xpd, i) {
+		if(!IS_SET(xpd->digital_outputs, i) && !IS_SET(xpd->digital_inputs, i))
+			len += sprintf(page + len, "%d ", priv->ring_sig[i]);
+	}
+#endif
 	len += sprintf(page + len, "\n\t%-17s: ", "battery");
 	for_each_line(xpd, i) {
-		len += sprintf(page + len, "%d ", IS_SET(priv->battery, i));
+		len += sprintf(page + len, "%2d ", IS_SET(priv->battery, i));
+	}
+	len += sprintf(page + len, "\n\t%-17s: ", "battery_debounce");
+	for_each_line(xpd, i) {
+		len += sprintf(page + len, "%2d ", priv->battery_debounce[i]);
 	}
 	len += sprintf(page + len, "\n");
 	spin_unlock_irqrestore(&xpd->lock, flags);
@@ -896,8 +990,7 @@ static int process_slic_cmdline(xpd_t *xpd, char *cmdline)
 static int proc_xpd_slic_write(struct file *file, const char __user *buffer, unsigned long count, void *data)
 {
 	xpd_t		*xpd = data;
-	const int	LINE_LEN = 500;
-	char		buf[LINE_LEN];
+	char		buf[MAX_PROC_WRITE];
 	char		*p;
 	int		i;
 	int		ret;
@@ -905,7 +998,7 @@ static int proc_xpd_slic_write(struct file *file, const char __user *buffer, uns
 	if(!xpd)
 		return -ENODEV;
 	for(i = 0; i < count; /* noop */) {
-		for(p = buf; p < buf + LINE_LEN; p++) {	/* read a line */
+		for(p = buf; p < buf + MAX_PROC_WRITE; p++) {	/* read a line */
 			if(i >= count)
 				break;
 			if(get_user(*p, buffer + i))
@@ -914,7 +1007,7 @@ static int proc_xpd_slic_write(struct file *file, const char __user *buffer, uns
 			if(*p == '\n' || *p == '\r')	/* whatever */
 				break;
 		}
-		if(p >= buf + LINE_LEN)
+		if(p >= buf + MAX_PROC_WRITE)
 			return -E2BIG;
 		*p = '\0';
 		ret = process_slic_cmdline(xpd, buf);
@@ -929,6 +1022,11 @@ static int proc_xpd_slic_write(struct file *file, const char __user *buffer, uns
 int __init card_fxo_startup(void)
 {
 	INFO("%s revision %s\n", THIS_MODULE->name, ZAPTEL_VERSION);
+#ifdef	SOFT_RING
+	INFO("FEATURE: %s with SOFT_RING\n", THIS_MODULE->name);
+#else
+	INFO("FEATURE: %s without SOFT_RING\n", THIS_MODULE->name);
+#endif
 	xproto_register(&PROTO_TABLE(FXO));
 	return 0;
 }
