@@ -8,7 +8,7 @@
 #include <usb.h>
 #include "hexfile.h"
 
-static const char rcsid[] = "$Id: fpga_load.c 1212 2006-07-08 00:43:31Z tzafrir $";
+static const char rcsid[] = "$Id: fpga_load.c 1457 2006-09-09 15:24:12Z tzafrir $";
 
 #define	ERR(fmt, arg...)	fprintf(stderr, "%s: ERROR: " fmt, progname, ## arg)
 #define	INFO(fmt, arg...)	fprintf(stderr, "%s: " fmt, progname, ## arg)
@@ -313,7 +313,7 @@ int send_hexline(struct usb_dev_handle *handle, struct hexline *hexline, int seq
 	phead->d.data_packet.seq = seq;
 	phead->d.data_packet.reserved = 0x00;
 	memcpy(phead->d.data_packet.data, data, len);
-	len += sizeof(phead);
+	len += sizeof(hexline->d.content.header);
 	if(verbose >= LOG_INFO)
 		INFO("%04d+\r", seq);
 	ret = usb_bulk_write(handle, MY_EP_OUT, (char *)phead, len, TIMEOUT);
@@ -324,6 +324,8 @@ int send_hexline(struct usb_dev_handle *handle, struct hexline *hexline, int seq
 		ERR("usb: bulk_write short write (%d)\n", ret);
 		return -EFAULT;
 	}
+	if (verbose >= LOG_DEBUG)
+		dump_packet((char*)phead, len);
 	ret = usb_bulk_read(handle, MY_EP_IN, buf, sizeof(buf), TIMEOUT);
 	if(ret < 0) {
 		ERR("usb: bulk_read failed (%d)\n", ret);
@@ -577,11 +579,13 @@ void usage()
 	fprintf(stderr, "\t\t[-d]		# Get device version from eeprom\n");
 	fprintf(stderr, "\t\t[-I <hexfile>]	# Input from <hexfile>\n");
 	fprintf(stderr, "\t\t[-g]		# Get eeprom from device\n");
+#ifdef XORCOM_INTERNAL
 	fprintf(stderr, "\t\t[-C srC byte]	# Set Address sourCe (default: C0)\n");
 	fprintf(stderr, "\t\t[-V vendorid]	# Set Vendor id on device\n");
 	fprintf(stderr, "\t\t[-P productid]	# Set Product id on device\n");
 	fprintf(stderr, "\t\t[-R release]	# Set Release. 2 dot separated decimals\n");
 	fprintf(stderr, "\t\t[-S serial]	# Set Serial. 8 comma separated numbers\n");
+#endif
 	exit(1);
 }
 
