@@ -32,6 +32,7 @@ void card_detected(struct card_desc_struct *card_desc);
 xpd_t *xpd_alloc(size_t privsize, xbus_t *xbus, int xpd_num, const xproto_table_t *proto_table, int channels, byte revision);
 void xpd_remove(xpd_t *xpd);
 void update_xpd_status(xpd_t *xpd, int alarm_flag);
+void update_zap_ring(xpd_t *xpd, int pos, bool on);
 void update_line_status(xpd_t *xpd, int pos, bool good);
 void fill_beep(u_char *buf, int duration);
 void xpp_tick(unsigned long param);
@@ -49,6 +50,6 @@ extern struct workqueue_struct		*xpp_worker;
 extern struct proc_dir_entry	*xpp_proc_toplevel;
 #endif
 
-#define	SPAN_REGISTERED(xpd)	((xpd)->span.flags & ZT_FLAG_REGISTERED)
+#define	SPAN_REGISTERED(xpd)	atomic_read(&(xpd)->zt_registered)
 
 #endif	/* XPP_ZAP_H */
