@@ -27,7 +27,7 @@
 #include "zap_debug.h"
 #include <linux/module.h>
 
-static const char rcsid[] = "$Id: xproto.c 1212 2006-07-08 00:43:31Z tzafrir $";
+static const char rcsid[] = "$Id: xproto.c 1457 2006-09-09 15:24:12Z tzafrir $";
 
 extern	int print_dbg;
 static int packet_process(xbus_t *xbus, int xpd_num, xpacket_t *pack);
@@ -298,18 +298,13 @@ int xproto_register(const xproto_table_t *proto_table)
 	CHECK_XOP(card_tick);
 	CHECK_XOP(card_zaptel_preregistration);
 	CHECK_XOP(card_zaptel_postregistration);
-#ifdef WITH_RBS
 	CHECK_XOP(card_hooksig);
-#else
-	CHECK_XOP(card_sethook);
-#endif
 	// CHECK_XOP(card_ioctl);	// optional method -- call after testing
 	CHECK_XOP(SYNC_SOURCE);
 	CHECK_XOP(PCM_WRITE);
 	CHECK_XOP(CHAN_ENABLE);
 	CHECK_XOP(CHAN_CID);
 	CHECK_XOP(RING);
-	CHECK_XOP(SETHOOK);
 	CHECK_XOP(RELAY_OUT);
 
 	xprotocol_tables[type] = proto_table;
