@@ -101,7 +101,7 @@ void xpd_set_addr(xpd_addr_t *addr, int xpd_num);
 	((p)->datalen + sizeof(xpd_addr_t) + 1)
 
 #define	XENTRY(card,op)					\
-	[ XPROTO_NAME(card,op) ] {			\
+	[ XPROTO_NAME(card,op) ] = {			\
 		.handler = XPROTO_HANDLER(card,op),	\
 		.datalen = RPACKET_DATALEN(card,op),	\
 		.name = #op,				\
@@ -151,20 +151,15 @@ struct xops {
 	int (*card_tick)(xbus_t *xbus, xpd_t *xpd);
 	int (*card_zaptel_preregistration)(xpd_t *xpd, bool on);
 	int (*card_zaptel_postregistration)(xpd_t *xpd, bool on);
-#ifdef	WITH_RBS
 	int (*card_hooksig)(xbus_t *xbus, xpd_t *xpd, int pos, zt_txsig_t txsig);
-#else
-	int (*card_sethook)(xbus_t *xbus, xpd_t *xpd, int pos, int hookstate);
-#endif
 	int (*card_ioctl)(xpd_t *xpd, int pos, unsigned int cmd, unsigned long arg);
 
 	int (*SYNC_SOURCE)(xbus_t *xbus, xpd_t *xpd, bool setit, bool is_master);
-	int (*PCM_WRITE)(xbus_t *xbus, xpd_t *xpd, xpp_line_t hookstate,  volatile byte *buf);
+	int (*PCM_WRITE)(xbus_t *xbus, xpd_t *xpd, xpp_line_t offhook,  volatile byte *buf);
 
 	int (*CHAN_ENABLE)(xbus_t *xbus, xpd_t *xpd, xpp_line_t lines, bool on);
 	int (*CHAN_CID)(xbus_t *xbus, xpd_t *xpd, int pos);
 	int (*RING)(xbus_t *xbus, xpd_t *xpd, int pos, bool on);
-	int (*SETHOOK)(xbus_t *xbus, xpd_t *xpd, int pos, bool offhook);
 	int (*RELAY_OUT)(xbus_t *xbus, xpd_t *xpd, byte which, bool on);
 };
 
