@@ -50,8 +50,11 @@ struct ztdeth_header {
 
 /* We take the raw message, put it in an ethernet frame, and add a
    two byte addressing header at the top for future use */
-
+#ifdef DEFINE_SPINLOCK
+static DEFINE_SPINLOCK(zlock);
+#else
 static spinlock_t zlock = SPIN_LOCK_UNLOCKED;
+#endif
 
 static struct ztdeth {
 	unsigned char addr[ETH_ALEN];
@@ -87,10 +90,10 @@ static int ztdeth_rcv(struct sk_buff *skb, struct net_device *dev, struct packet
 	struct ztdeth_header *zh;
 	zh = (struct ztdeth_header *)skb->nh.raw;
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,9)
-        span = ztdeth_getspan(eth_hdr(skb)->h_source, zh->subaddr);
+	span = ztdeth_getspan(eth_hdr(skb)->h_source, zh->subaddr);
 #else
 	span = ztdeth_getspan(skb->mac.ethernet->h_source, zh->subaddr);
-#endif
+#endif	
 	if (span) {
 		skb_pull(skb, sizeof(struct ztdeth_header));
 		zt_dynamic_receive(span, (unsigned char *)skb->data, skb->len);
