@@ -6,6 +6,7 @@
  * Written by Robert Pleh <robert.pleh@hermes.si>
  * 2.6 version by Tony Hoyle
  * Unified by Mark Spencer <markster@digium.com>
+ * Converted to use RTC on i386 by Tony Mountifield <tony@softins.co.uk>
  *
  * Copyright (C) 2002, Hermes Softlab
  * Copyright (C) 2004, Digium, Inc.
@@ -38,6 +39,21 @@
 #  error "This kernel is too old: not supported by this file"
 #endif
 
+/*
+ * NOTE: (only applies to kernel 2.6)
+ * If using an i386 architecture without a PC real-time clock,
+ * the #define USE_RTC should be commented out.
+ */
+#if defined(__i386__) || defined(__x86_64__)
+#if LINUX_VERSION_CODE >= VERSION_CODE(2,6,13)
+#define USE_RTC
+#else
+#if 0
+#define USE_RTC
+#endif
+#endif
+#endif
+
 #include <linux/kernel.h>
 #include <linux/errno.h>
 #include <linux/module.h>
@@ -54,6 +70,9 @@
 #include <asm/io.h>
 #endif
 #ifdef LINUX26
+#ifdef USE_RTC
+#include <linux/rtc.h>
+#endif
 #include <linux/moduleparam.h>
 #endif
 #include "ztdummy.h"
@@ -77,8 +96,13 @@ static struct ztdummy *ztd;
 static int debug = 0;
 
 #ifdef LINUX26
+#ifndef USE_RTC
 /* New 2.6 kernel timer stuff */
 static struct timer_list timer;
+#if HZ != 1000
+#warning This module will not be usable since the kernel HZ setting is not 1000 ticks per second.
+#endif
+#endif
 #else
 #if LINUX_VERSION_CODE < VERSION_CODE(2,4,5)
 #  error "This kernel is too old: not supported by this file"
@@ -103,6 +127,28 @@ extern uhci_t **uhci_devices;
 
 
 #ifdef LINUX26
+#ifdef USE_RTC
+/* rtc_interrupt - called at 1024Hz from hook in RTC handler */
+static void ztdummy_rtc_interrupt(void *private_data)
+{
+	struct ztdummy *ztd = private_data;
+	unsigned int ticks;
+
+	atomic_inc(&ztd->ticks);
+	ticks = atomic_read(&ztd->ticks);
+	if (ticks == 42 || ticks == 85) {
+		/* skip it */
+	} else if (ticks >= 128) {
+		/* skip and restart count */
+		atomic_set(&ztd->ticks, 0);
+	} else {
+		/* zaptel timing - called in 125 of every 128 interrupts = 1000Hz */
+		zt_receive(&ztd->span);
+		zt_transmit(&ztd->span);
+	}
+}
+#else
+/* use kernel system tick timer if PC architecture RTC is not available */
 static void ztdummy_timer(unsigned long param)
 {
     zt_receive(&ztd->span);
@@ -110,6 +156,7 @@ static void ztdummy_timer(unsigned long param)
     timer.expires = jiffies + 1;
     add_timer(&timer);
 }
+#endif
 #else
 static void ztdummy_interrupt(int irq, void *dev_id, struct pt_regs *regs)
 {
@@ -150,9 +197,17 @@ static int ztdummy_initialize(struct ztdummy *ztd)
 
 int init_module(void)
 {
-#ifndef LINUX26
+#ifdef LINUX26
+#ifdef USE_RTC
+    int err;
+#endif
+#else
     int irq;
+#ifdef DEFINE_SPINLOCK
+    DEFINE_SPINLOCK(mylock);
+#else
     spinlock_t mylock = SPIN_LOCK_UNLOCKED;
+#endif
 	
     if (uhci_devices==NULL){
         printk ("ztdummy: Uhci_devices pointer error.\n");
@@ -165,6 +220,13 @@ int init_module(void)
     }
 #endif
 
+#if defined(LINUX26) && !defined(USE_RTC)
+    if (HZ != 1000) {
+	    printk("ztdummy: This module requires the kernel HZ setting to be 1000 ticks per second\n");
+	    return -ENODEV;
+    }
+#endif /* defined(LINUX26) && !defined(USE_RTC) */
+
     ztd = kmalloc(sizeof(struct ztdummy), GFP_KERNEL);
     if (ztd == NULL) {
 	    printk("ztdummy: Unable to allocate memory\n");
@@ -180,10 +242,25 @@ int init_module(void)
     }
 
 #ifdef LINUX26
+#ifdef USE_RTC
+    atomic_set(&ztd->ticks, 0);
+    ztd->rtc_task.func = ztdummy_rtc_interrupt;
+    ztd->rtc_task.private_data = ztd;
+    err = rtc_register(&ztd->rtc_task);
+    if (err < 0) {
+	printk("ztdummy: Unable to register zaptel rtc driver\n");
+        zt_unregister(&ztd->span);
+	kfree(ztd);
+	return err;
+    }
+    rtc_control(&ztd->rtc_task, RTC_IRQP_SET, 1024);	/* 1024 Hz */
+    rtc_control(&ztd->rtc_task, RTC_PIE_ON, 0);
+#else
     init_timer(&timer);
     timer.function = ztdummy_timer;
     timer.expires = jiffies + 1;
     add_timer(&timer);
+#endif
 #else
     irq=s->irq;
     spin_lock_irq(&mylock);
@@ -214,7 +291,12 @@ int init_module(void)
 void cleanup_module(void)
 {
 #ifdef LINUX26
+#ifdef USE_RTC
+    rtc_control(&ztd->rtc_task, RTC_PIE_OFF, 0);
+    rtc_unregister(&ztd->rtc_task);
+#else
     del_timer(&timer);
+#endif
 #else
     free_irq(s->irq, ztd);  /* disable interrupts */
 #endif
